function [u,U,iter,time] = mympc(A,B,Q,R,P,N,umin,umax,xmin,xmax,x,solver,z0)
% solves mpc-problem in condensed form
%
% Inputs:
%  A,B       - system matrices
%  Q,R,P,N   - MPC tuning parameters
%  umin,umax - box constraints for inputs
%  xmin,xmax - box constraints for states (specify -inf,inf to run without state constraints)
%  x         - current state of the plant
%  solver    - chosen solver to solve the mpc problem
%  z0        - starting point for some solvers
%
% Outputs:
%  u      - first input to be applied to the plant
%  U      - all inputs over the whole horizon
%  iter   - number of needed iterations
%  time   - tic-toc-time for calling solver 
%
% Copyright (C) embotech GmbH, Zurich, Switzerland, 2015.

%% calculate QP matrices
if strcmp(solver,'quadprog') || strcmp(solver,'mygm')

    % calculate the matrices for the condensed form

    Aii=eye(size(A));
    AA=Aii;
    for i=1:N
        Aii=A*Aii;
        AA=[AA;Aii];
    end

    AiB=B;
    BB=kron(eye(N),AiB);
    for i=1:N-1;
        AiB=A*AiB;
        BB=BB+kron(diag(ones(N-i,1),-i),AiB);
    end
    BB=[zeros(size(B,1),size(BB,2)); BB];

    QQ=blkdiag(kron(eye(N),Q),P);
    RR=kron(eye(N),R);

    H=2*(RR + BB'*QQ*BB);
    H=(H+H')/2; %make symmetric (to avoid warnings in quadprog)
    F=2*AA'*QQ*BB;
    f=F'*x;

    % input constraints
    lb = repmat(umin,N,1);
    ub = repmat(umax,N,1);

    % state constraints
    Aineq=[BB; -BB];
    bmaxmin=[repmat(xmax,N+1,1);-repmat(xmin,N+1,1)];
    bineq =bmaxmin +[-AA;AA]*x;
    % delete all rows for infinite xmax/xmin
    idx=isinf(bmaxmin);
    Aineq(idx,:)=[];
    bineq(idx,:)=[];
    
elseif strcmp(solver,'myadmm')
    
    % calculate the matrices for the non-condensed form
   
    H   = blkdiag(kron(eye(N-1),2*blkdiag(R,Q))); 
    H   = blkdiag(H,2*blkdiag(R,P)); % add terminal cost
    f   = zeros(size(H,1),1);
    Aeq = kron(eye(N),[B -eye(size(A,1))]) + kron(diag(ones(N-1,1),-1),[zeros(size(B)), A]);
    beq = zeros(size(Aeq,1),1);
    beq(1:size(A,1)) = -A*x;
    lb = repmat([umin;xmin],N,1);
    ub = repmat([umax;xmax],N,1);
    
end


%% solve the problem

hasStateCon = any(isfinite(xmin))||any(isfinite(xmax));

switch solver
    %Note: each solver should assign the variables U, time, iter.
    
    case 'quadprog'
        options=optimset;
        options.Algorithm='active-set'; %to get rid of some warnings.
        options.Display='off';
        tic;
        U = quadprog(H,f,Aineq,bineq,[],[],lb,ub,[],options);
        time=toc;
        iter=nan;
        
        
    case 'mygm'
        if hasStateCon
            error('solver not applicable in case of state constraints.');
        end
    
        tic;
        %TO ADD: call "mygm"       
        time=toc;      
        
 
    case 'myadmm'   
        tic;
        %TO ADD: call "myadmm"   
        time=toc;         
        
        
    case 'myforcespro'      
        %assign parametric data
        problem.minusA_times_x0 = -A*x;
        %TO ADD: call controller 
        %U=  %TO ADD
        %iter=  %TO ADD
        %time=  %TO ADD
        
        
    otherwise
        error('unknown solver.');
end

u=U(1:size(B,2));

end
